import { AxiosRequestConfig } from 'axios'
import { ElMessageBox, FormInstance } from 'element-plus'
import { copyObjectValues, copySameKeys, isObjEqual, paramsHandling, removeFalsyKeys, rowKeyHandling } from '../utils/tools'
import { CrudContext } from './index'
import { CrudElType } from '@/vite-env'
import { Ref } from 'vue'

interface HookParams {
    CrudCtx: CrudContext
    Data: Record<string, any>
}

/** 全局配置类 */
export class CrudFunction {
    /** 请求方法 */
    protected request: <T>(config: AxiosRequestConfig) => Promise<T>

    /** 初始化 */
    constructor() {
        this.request = <T>(config: AxiosRequestConfig): Promise<T> => {
            return this.CrudCtx().componentConfig.axiosService.request(config)
        }
    }

    /** 获取容器实例 */
    protected CrudCtx = () => {
        return CrudContext.getCrudCtx()
    }

    /** 查询之前，从dataStorage中取出数据，处理成接口所需要的属性，并返回出来 */
    beforeQueryHandler = ({ CrudCtx, Data }: HookParams): boolean | Record<string, any> => {
        this.CrudCtx().componentConfig.TableStore.loading = true
        // 对参数的空值处理为undefined（表单点击删除按钮后，会把值写成空字符串或者null）
        return { ...removeFalsyKeys(Data) }
    }
    /** 查询 */
    query = () => {
        const Data = this.beforeQueryHandler({ CrudCtx: this.CrudCtx(), Data: this.CrudCtx().dataStorage.searchModel })

        if (Data === false || typeof Data === 'boolean') return

        this.CrudCtx().CrudDebug('【查询参数】\n', { Data })
        this.request<CrudElType.ResData>({
            url: this.CrudCtx().componentConfig.ApiNames.query,
            method: 'post',
            data: {
                pagenum: this.CrudCtx().componentConfig.PageStore.current,
                pagesize: this.CrudCtx().componentConfig.PageStore.pageSize,
                ...Data
            }
        })
            .then((Res) => {
                this.afterQueryHandler(Res)
            })
            .finally(() => {
                this.CrudCtx().componentConfig.TableStore.loading = false
            })
    }
    /** 查询之后 */
    afterQueryHandler = (ResData: CrudElType.ResData) => {
        this.CrudCtx().CrudDebug('【查询之后】\n', '把查询到的数据、分页信息进行赋值\n', { ResData })
        const {
            data,
            page: { pageNum, pageSize, total }
        } = ResData

        this.CrudCtx().dataStorage.originalData.value = data
        this.CrudCtx().dataStorage.tableData.value = data
        Object.assign(this.CrudCtx().componentConfig.PageStore, { current: pageNum, pageSize: pageSize, total: total })
    }

    /** 点击新增之前 */
    beforeAddHandler = ({ CrudCtx, Data }: HookParams): boolean => true
    /** 点击新增 */
    add = () => {
        if (!this.beforeAddHandler({ CrudCtx: this.CrudCtx(), Data: this.CrudCtx().dataStorage.editModel })) return

        this.afterAddHandler()
    }
    /** 点击新增之后 */
    afterAddHandler = () => {
        this.CrudCtx().dataStorage.Mode.value = 'add'
        this.CrudCtx().dataStorage.drawerVisible.value = true
    }

    /** 点击编辑之前 */
    beforeEditHandler = ({ CrudCtx, Data, Row }: HookParams & { Row: Record<string, any> }): boolean => true
    /** 点击编辑 */
    edit = (row: Record<string, any>) => {
        this.CrudCtx().dataStorage.editFormValue = row
        const editModelRow = copySameKeys(this.CrudCtx().dataStorage.editModel, row)

        const Data = Object.assign(this.CrudCtx().dataStorage.editModel, editModelRow)

        if (!this.beforeEditHandler({ CrudCtx: this.CrudCtx(), Data, Row: row })) return

        this.afterEditHandler()
    }
    /** 点击编辑之后 */
    afterEditHandler = () => {
        this.CrudCtx().dataStorage.Mode.value = 'edit'
        this.CrudCtx().dataStorage.drawerVisible.value = true
    }

    /** 点击删除之前 */
    beforeRemoveHandler = ({ CrudCtx, Data }: HookParams): boolean | Record<string, any> => {
        const key = this.CrudCtx().componentConfig.PropsStore.rowKeys! || this.CrudCtx().componentConfig.PropsStore.rowKey! || false

        if (!key) {
            console.error('错误的rowKey:', this.CrudCtx().componentConfig.PropsStore)
            return false
        }

        return {
            ...rowKeyHandling(Data, key),
            row: Data
        }
    }
    // 因为delete是保留参数，所有定义成remove
    remove = (row: Record<string, any>) => {
        const Data = this.beforeRemoveHandler({ CrudCtx: this.CrudCtx(), Data: row })
        if (!Data) return

        ElMessageBox.confirm('确定要删除这条数据？', '删除数据 提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning',
            center: true
        }).then(() => {
            this.request<CrudElType.Res>({
                url: this.CrudCtx().componentConfig.ApiNames.delete,
                method: 'post',
                data: Data
            }).then((Res) => {
                this.afterRemoveHandler(Res)
            })
        })
    }
    /** 删除之后 */
    afterRemoveHandler = (ResData: CrudElType.Res) => {
        this.CrudCtx().CrudDebug('【删除之后】\n', '成功删除后执行刷新函数\n', { ResData })
        this.query()
    }

    /** 标记删除之前 */
    beforeDeleteFlagHandler = ({ CrudCtx, Data }: HookParams): boolean | Record<string, any> => {
        this.CrudCtx().CrudDebug('【标记删除之前】\n', '把传入的当前点击行数处理标记成删除接口所需的样子返回\n')
        const key = this.CrudCtx().componentConfig.PropsStore.rowKeys! || this.CrudCtx().componentConfig.PropsStore.rowKey! || false
        if (!key) {
            console.error('错误的rowKey:', this.CrudCtx().componentConfig.PropsStore)
            return false
        }

        return {
            ...rowKeyHandling(Data, key),
            deleteFlag: Data.deleteFlag == 0 ? 1 : 0,
            row: Data
        }
    }
    deleteFlag = (row: Record<string, any>) => {
        this.CrudCtx().CrudDebug('【点击标记删除】\n', '把当前点击行数据传入删除前钩子进行数据处理操作\n')
        const Data = this.beforeDeleteFlagHandler({ CrudCtx: this.CrudCtx(), Data: row })
        if (Data === false || typeof Data === 'boolean') return

        this.CrudCtx().CrudDebug('【标记删除中】\n', '使用返回的参数发起删除请求\n', { Data })
        ElMessageBox.confirm('确定要修改这条数据的标记状态吗？', '修改数据状态 提示', {
            confirmButtonText: '确定',
            cancelButtonText: '取消',
            type: 'warning',
            center: true
        }).then(() => {
            this.request<CrudElType.Res>({
                url: this.CrudCtx().componentConfig.ApiNames.deleteFlag,
                method: 'post',
                data: Data
            }).then((Res) => {
                this.afterDeleteFlagHandler(Res)
            })
        })
    }
    /** 标记删除之后 */
    afterDeleteFlagHandler = (ResData: CrudElType.Res) => {
        this.CrudCtx().CrudDebug('【标记删除之后】\n', '成功删除后执行刷新函数\n', ResData)
        this.query()
    }

    beforeHandleSubmit = (): boolean => true
    /** 处理新增/编辑提交事件 */
    handleSubmit = async (formEl: FormInstance | undefined) => {
        if (!formEl) return
        await formEl.validate((valid, fields) => {
            if (valid) {
                if (!this.beforeHandleSubmit()) return

                this.CrudCtx().dataStorage.Mode.value === 'add' ? this.handleAdd() : this.handleEdit()
            }
        })
    }

    /** 新增请求发起之前 */
    beforeHandleAddHandler = ({ CrudCtx, Data }: HookParams): boolean | Record<string, any> => {
        this.CrudCtx().CrudDebug('【新增请求发起之前】\n', '对表单数据进行自定义处理后返回\n')

        return Data
    }
    handleAdd = () => {
        this.CrudCtx().CrudDebug('【点击发起新增请求】\n', '把当前编辑表单传入新增请求发起之前钩子\n')
        const Data = this.beforeHandleAddHandler({ CrudCtx: this.CrudCtx(), Data: this.CrudCtx().dataStorage.editModel })
        if (Data === false || typeof Data === 'boolean') return

        this.CrudCtx().CrudDebug('【发起新增请求中】\n', '使用返回的参数发起请求\n', { Data })
        this.request<CrudElType.Res>({
            url: this.CrudCtx().componentConfig.ApiNames.add,
            method: 'post',
            data: Data
        }).then((Res) => {
            this.afterHandleAddHandler(Res)
        })
    }
    /** 新增请求发起之后 */
    afterHandleAddHandler = (ResData: CrudElType.Res) => {
        this.CrudCtx().CrudDebug('【新增请求发起之后】\n', '成功后关闭新增表单, 执行刷新操作\n', { ResData })
        this.cancelForm()
        this.query()
    }

    /** 编辑请求发起之前 */
    beforeHandleEditHandler = ({ CrudCtx, Data }: HookParams): boolean | Record<string, any> => {
        this.CrudCtx().CrudDebug('【编辑请求发起之前】\n', '数据编辑表单数据, 只提交修改项, 要有rowKey\n')

        const key = this.CrudCtx().componentConfig.PropsStore.rowKeys || this.CrudCtx().componentConfig.PropsStore.rowKey || false
        if (!key) {
            console.error('错误的rowKey:', this.CrudCtx().componentConfig.PropsStore)
            return false
        }

        return {
            ...isObjEqual(this.CrudCtx().dataStorage.editFormValue, Data),
            ...rowKeyHandling(Data, key),
            row: Data
        }
    }
    /** 处理表单编辑动作 */
    handleEdit = () => {
        this.CrudCtx().CrudDebug('【点击发起编辑请求】\n', '把当前编辑表单传入编辑请求发起之前钩子\n')
        const Data = this.beforeHandleEditHandler({ CrudCtx: this.CrudCtx(), Data: this.CrudCtx().dataStorage.editModel })
        if (Data === false || typeof Data === 'boolean') return

        this.CrudCtx().CrudDebug('【发起编辑请求中】\n', '使用返回的参数发起请求\n', { Data })
        this.request<CrudElType.Res>({
            url: this.CrudCtx().componentConfig.ApiNames.edit,
            method: 'post',
            data: Data
        }).then((Res) => {
            this.afterHandleEditHandler(Res)
        })
    }
    /** 编辑请求发起之后 */
    afterHandleEditHandler = (ResData: CrudElType.Res) => {
        this.CrudCtx().CrudDebug('【编辑请求发起之后】\n', '成功后关闭编辑表单, 执行刷新操作\n', { ResData })
        this.cancelForm()
        this.query()
    }

    /** 处理新增/编辑重置事件 */
    resetForm = (Mode: string, formEl: Ref<any>) => {
        this.CrudCtx().CrudDebug('【处理新增/编辑重置事件】\n', { Mode, formEl })
        if (!formEl) return
        formEl.value.resetFields()
    }

    /** 处理新增/编辑取消事件 */
    cancelForm = () => {
        this.CrudCtx().CrudDebug('【处理新增/编辑取消事件】\n', '默认关闭编辑框、重置编辑表单参数\n')
        this.CrudCtx().dataStorage.drawerVisible.value = false

        Object.assign(this.CrudCtx().dataStorage.editModel, this.CrudCtx().componentConfig.initEditModel)
    }
}
